@@ -1,6 +1,8 @@
 import multiprocessing as mp
 import os
 import sys
+import json
+from dataclasses import dataclass
 
 import psutil
 import platform
@@ -14,6 +16,7 @@ from pathlib import Path
 from typing import List, Optional, Dict, Set
 from urllib.parse import ParseResult as URL
 
+from rlbot.gamelaunch.epic_launch import launch_with_epic_login_trick, launch_with_epic_simple
 from rlbot.utils.structures import game_data_struct
 
 from rlbot import gateway_util
@@ -61,6 +64,17 @@ class ROCKET_LEAGUE_PROCESS_INFO:
         return [arg.replace(ROCKET_LEAGUE_PROCESS_INFO.PORT_PLACEHOLDER, str(port))
                 for arg in ROCKET_LEAGUE_PROCESS_INFO.IDEAL_ARGS]
 
+@dataclass
+class RocketLeagueLauncherPreference:
+    STEAM = 'steam'
+    EPIC = 'epic'
+    preferred_launcher: str
+    use_login_tricks: bool
+
+
+# By default, we will attempt Epic with no login tricks, then fall back to Steam.
+DEFAULT_LAUNCHER_PREFERENCE = RocketLeagueLauncherPreference(RocketLeagueLauncherPreference.EPIC, False)
+
 
 @contextmanager
 def setup_manager_context():
@@ -149,7 +163,7 @@ class SetupManager:
 
         return is_rocket_league_running
 
-    def connect_to_game(self):
+    def connect_to_game(self, launcher_preference: RocketLeagueLauncherPreference = DEFAULT_LAUNCHER_PREFERENCE):
         """
         Connects to the game by initializing self.game_interface.
         """
@@ -169,7 +183,7 @@ class SetupManager:
         # Launch the game if it is not running.
         elif not self.is_rocket_league_running(port):
             mergeTASystemSettings()
-            self.launch_rocket_league(port=port)
+            self.launch_rocket_league(port=port, launcher_preference=launcher_preference)
 
         try:
             self.game_interface.load_interface()
@@ -188,12 +202,21 @@ class SetupManager:
                 return int(rocket_league_port)
         return None
 
-    def launch_rocket_league(self, port):
+    def launch_rocket_league(self, port, launcher_preference: RocketLeagueLauncherPreference = DEFAULT_LAUNCHER_PREFERENCE):
         """
         Launches Rocket League but does not connect to it.
         """
         ideal_args = ROCKET_LEAGUE_PROCESS_INFO.get_ideal_args(port)
-        self.logger.info(f'Launching Rocket League with args: {ideal_args}')
+
+        if launcher_preference.preferred_launcher == RocketLeagueLauncherPreference.EPIC:
+            if launcher_preference.use_login_tricks:
+                if launch_with_epic_login_trick(ideal_args):
+                    return
+                else:
+                    self.logger.info("Epic login trick seems to have failed, falling back to simple Epic launch.")
+            # Fall back to simple if the tricks failed or we opted out of tricks.
+            if launch_with_epic_simple(ideal_args):
+                return
 
         # Try launch via Steam.
         steam_exe_path = try_get_steam_executable_path()
@@ -203,12 +226,11 @@ class SetupManager:
                 '-applaunch',
                 str(ROCKET_LEAGUE_PROCESS_INFO.GAMEID),
             ] + ideal_args
+            self.logger.info(f'Launching Rocket League with: {exe_and_args}')
             _ = subprocess.Popen(exe_and_args)  # This is deliberately an orphan process.
             return
 
-        # TODO: Figure out launching via Epic games
-
-        self.logger.warning('Using fall-back launch method.')
+        self.logger.warning(f'Launching Rocket League using Steam-only fall-back launch method with args: {ideal_args}')
         self.logger.info("You should see a confirmation pop-up, if you don't see it then click on Steam! "
                          'https://gfycat.com/AngryQuickFinnishspitz')
         args_string = '%20'.join(ideal_args)
@@ -222,12 +244,8 @@ class SetupManager:
 
             try:
                 _ = subprocess.Popen(linux_args)
-
             except OSError:
-                self.logger.warning(
-                    'Could not find Steam executable, using browser to open instead.')
-            else:
-                return
+                self.logger.warning('Could not find Steam executable, using browser to open instead.')
 
         try:
             webbrowser.open(f'steam://rungameid/{ROCKET_LEAGUE_PROCESS_INFO.GAMEID}//{args_string}')
@@ -643,13 +661,15 @@ def try_get_steam_executable_path() -> Optional[Path]:
     try:
         from winreg import OpenKey, HKEY_CURRENT_USER, ConnectRegistry, QueryValueEx, REG_SZ
     except ImportError as e:
-        return None  # TODO: Linux support.
+        return  # TODO: Linux support.
 
     try:
         key = OpenKey(ConnectRegistry(None, HKEY_CURRENT_USER), r'Software\Valve\Steam')
         val, val_type = QueryValueEx(key, 'SteamExe')
     except FileNotFoundError:
-        return None
+        return
     if val_type != REG_SZ:
-        return None
+        return
     return Path(val)
+
+
